"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const eas_build_job_1 = require("@expo/eas-build-job");
const path_1 = tslib_1.__importDefault(require("path"));
const api_1 = require("../../api");
const repository_1 = require("../../build/utils/repository");
const EasCommand_1 = tslib_1.__importDefault(require("../../commandUtils/EasCommand"));
const getProjectIdAsync_1 = require("../../commandUtils/context/contextUtils/getProjectIdAsync");
const SetUpBuildCredentialsCommandAction_1 = require("../../credentials/manager/SetUpBuildCredentialsCommandAction");
const generated_1 = require("../../graphql/generated");
const UserPreferencesMutation_1 = require("../../graphql/mutations/UserPreferencesMutation");
const AppQuery_1 = require("../../graphql/queries/AppQuery");
const log_1 = tslib_1.__importStar(require("../../log"));
const git_1 = require("../../onboarding/git");
const installDependencies_1 = require("../../onboarding/installDependencies");
const expoConfig_1 = require("../../project/expoConfig");
const prompts_1 = require("../../prompts");
const git_2 = tslib_1.__importDefault(require("../../vcs/clients/git"));
class Onboarding extends EasCommand_1.default {
    async runAsync() {
        const { args: { TARGET_PROJECT_DIRECTORY: targetProjectDirInput }, } = await this.parse(_a);
        const { loggedIn: { actor, graphqlClient }, analytics, } = await this.getContextAsync(_a, {
            nonInteractive: false,
        });
        if (actor.__typename === 'Robot') {
            throw new Error('This command is not available for robot users. Make sure you are not using a robot token and try again.');
        }
        if (!actor.preferences.onboarding) {
            throw new Error('This command can only be run as part of the onboarding process started on the Expo website. Visit https://expo.new to start a new project.');
        }
        if (!actor.preferences.onboarding.platform) {
            throw new Error('This command can only be run as part of the onboarding process started on the Expo website. It seems like you started an onboarding process, but we are missing some information needed to be filled in before running the eas init:onboarding command (selected platform). Continue the onboarding process on the Expo website.');
        }
        if (!actor.preferences.onboarding.environment) {
            throw new Error('This command can only be run as part of the onboarding process started on the Expo website. It seems like you started an onboarding process, but we are missing some information needed to be filled in before running the eas init:onboarding command (selected environment). Continue the onboarding process on the Expo website.');
        }
        if (!actor.preferences.onboarding.deviceType) {
            throw new Error('This command can only be run as part of the onboarding process started on the Expo website. It seems like you started an onboarding process, but we are missing some information needed to be filled in before running the eas init:onboarding command (selected device type). Continue the onboarding process on the Expo website.');
        }
        if (new Date(actor.preferences.onboarding.lastUsed) < new Date(Date.now() - 1000 * 60 * 60 * 24)) {
            log_1.default.warn('It seems like you started an onboarding process, but it has been a while since you last used it. If you want to start a new onboarding process, visit https://expo.new.');
            log_1.default.log();
        }
        const platform = actor.preferences.onboarding.platform === generated_1.AppPlatform.Android
            ? eas_build_job_1.Platform.ANDROID
            : eas_build_job_1.Platform.IOS;
        const app = await AppQuery_1.AppQuery.byIdAsync(graphqlClient, actor.preferences.onboarding.appId);
        const githubUsername = app.githubRepository
            ? app.githubRepository.metadata.githubRepoOwnerName
            : 'expo';
        const githubRepositoryName = app.githubRepository
            ? app.githubRepository.metadata.githubRepoName
            : 'expo-default-template';
        log_1.default.log(`👋 Welcome to Expo, ${actor.username}!`);
        log_1.default.log();
        log_1.default.log('✨ We will continue your onboarding process in EAS CLI');
        log_1.default.log();
        log_1.default.log(`🚚 Let's start by cloning ${app.githubRepository
            ? `your project (${githubUsername}/${githubRepositoryName})`
            : `default Expo template project (${githubUsername}/${githubRepositoryName})`} from GitHub and installing dependencies.`);
        log_1.default.log();
        let initialTargetProjectDirectory;
        if (targetProjectDirInput) {
            initialTargetProjectDirectory = targetProjectDirInput;
            log_1.default.log(`📂 Cloning the project to ${initialTargetProjectDirectory}`);
        }
        else {
            const { selectedTargetProjectDirectory } = await (0, prompts_1.promptAsync)({
                type: 'text',
                name: 'selectedTargetProjectDirectory',
                message: '📂 Where would you like to clone the project to?',
                initial: path_1.default.join(process.cwd(), githubRepositoryName),
            });
            initialTargetProjectDirectory = selectedTargetProjectDirectory;
        }
        log_1.default.log();
        const { targetProjectDir: finalTargetProjectDirectory } = await (0, git_1.runGitCloneAsync)({
            githubUsername,
            githubRepositoryName,
            targetProjectDir: initialTargetProjectDirectory,
        });
        const vcsClient = new git_2.default(finalTargetProjectDirectory);
        await (0, installDependencies_1.installDependenciesAsync)({
            projectDir: finalTargetProjectDirectory,
        });
        await vcsClient.trackFileAsync('package-lock.json');
        const shouldSetupCredentials = actor.preferences.onboarding.deviceType === generated_1.OnboardingDeviceType.Device &&
            actor.preferences.onboarding.environment === generated_1.OnboardingEnvironment.DevBuild;
        if (shouldSetupCredentials) {
            log_1.default.log('🔑 Now we need to set up build credentials for your project:');
            await new SetUpBuildCredentialsCommandAction_1.SetUpBuildCredentialsCommandAction(actor, graphqlClient, vcsClient, analytics, await getPrivateExpoConfigWithProjectIdAsync({
                projectDir: finalTargetProjectDirectory,
                graphqlClient,
                actor,
            }), getDynamicPrivateProjectConfigGetter({
                projectDir: finalTargetProjectDirectory,
                graphqlClient,
                actor,
            }), platform, 'development', finalTargetProjectDirectory).runAsync();
        }
        if (await vcsClient.hasUncommittedChangesAsync()) {
            log_1.default.log('📦 We will now commit the changes made by the configuration process and push them to GitHub:');
            log_1.default.log();
            log_1.default.log('🔍 Checking for changes in the repository...');
            await vcsClient.showChangedFilesAsync();
            await (0, repository_1.reviewAndCommitChangesAsync)(vcsClient, `[eas-onboarding] Install dependencies${shouldSetupCredentials ? ' and set up build credentials' : ''}`, { nonInteractive: false });
            log_1.default.log('📤 Pushing changes to GitHub...');
            await (0, git_1.runGitPushAsync)({
                targetProjectDir: finalTargetProjectDirectory,
            });
        }
        const { __typename, ...previousPreferences } = actor.preferences.onboarding;
        await UserPreferencesMutation_1.UserPreferencesMutation.markCliDoneInOnboardingUserPreferencesAsync(graphqlClient, {
            ...previousPreferences,
            appId: app.id,
        });
        log_1.default.log();
        log_1.default.log('🎉 We finished configuring your project.');
        log_1.default.log();
        log_1.default.log('🚀 You can now go back to the website to continue:');
        const url = new URL(`/onboarding/develop/set-up-project-on-your-machine?project=${app.slug}&accountId=${app.ownerAccount.id}`, (0, api_1.getExpoWebsiteBaseUrl)()).toString();
        log_1.default.log(`👉 ${(0, log_1.link)(url)}`);
    }
}
_a = Onboarding;
Onboarding.hidden = true;
Onboarding.aliases = ['init:onboarding', 'onboarding'];
Onboarding.description = 'continue onboarding process started on the expo.dev website';
Onboarding.flags = {};
Onboarding.args = [{ name: 'TARGET_PROJECT_DIRECTORY' }];
Onboarding.contextDefinition = {
    ..._a.ContextOptions.LoggedIn,
    ..._a.ContextOptions.Analytics,
};
exports.default = Onboarding;
// we can't get this automated by using command context because when we run a command the project directory doesn't exist yet
async function getPrivateExpoConfigWithProjectIdAsync({ projectDir, graphqlClient, actor, options, }) {
    const expBefore = (0, expoConfig_1.getPrivateExpoConfig)(projectDir, options);
    const projectId = await (0, getProjectIdAsync_1.validateOrSetProjectIdAsync)({
        exp: expBefore,
        graphqlClient,
        actor,
        options: {
            nonInteractive: false,
        },
    });
    const exp = (0, expoConfig_1.getPrivateExpoConfig)(projectDir, options);
    return {
        exp,
        projectId,
    };
}
// we can't get this automated by using command context because when we run a command the project directory doesn't exist yet
function getDynamicPrivateProjectConfigGetter({ projectDir, graphqlClient, actor, }) {
    return async (options) => {
        return {
            ...(await getPrivateExpoConfigWithProjectIdAsync({
                projectDir,
                graphqlClient,
                actor,
                options,
            })),
            projectDir,
        };
    };
}
